function [Opt_loc,Opt_av,track] = part_advection(video,PR)
%%%%%%%%%%%%%%%%%%%%
% This function computes the optical flow and with it the particle
% advection for an input video.
% INPUT:  video: the video to determine particle advection from
%         PR: particle roughness
% OUTPUT: track: The tracks of the particles in a N_particles x 2 x N_frames
% size



%%%%%%%%%%%%%%%%%%%%
%% compute optical flow (Piotr's Master Toolbox)
disp('calculating optical flow of video')
Vopt = zeros(size(video,1),size(video,2),size(video,4),2); %preallocate optical flow matrix [x-coordinate,y-coordinate,frame,optical flow direction (1 = x, 2 = y)]
for i = 2:size(video,4) %for every frame
    if rem(i,10) == 0
        disp(['frame ',num2str(i),' of ',num2str(size(video,4))]);
    end
    I1 = video(:,:,:,i-1);
    I2 = video(:,:,:,i);
    [Vx,Vy] = opticalFlow(rgb2gray(I1),rgb2gray(I2),'smooth',1,'radius',10,'type','LK'); %use Piotr,s optical flow
    Vopt(:,:,i,1) = Vy;
    Vopt(:,:,i,2) = Vx;
end

%% compute particle advection
disp('calculating particle advection')

[X,Y] = meshgrid(PR:PR:size(video,1)-PR,PR:PR:size(video,2)-PR); %initial particle positions
[Xint,Yint] = meshgrid(PR:PR:size(video,1)-PR,PR:PR:size(video,2)-PR);
Opt_av = zeros(size(X,1),size(X,2),size(video,4),2);%preallocating
w_av = fspecial('gaussian',[PR+1,PR+1]); %filter for calculating the weighted average optical flow of a particle
Opt_loc= zeros(size(X,1),size(X,2),size(video,4),2);%preallocating

%initialize
X2 = reshape(X,[1,size(X,1)*size(X,2)]);
Y2 = reshape(Y,[1,size(Y,1)*size(Y,2)]);
track = zeros(length(X2),2,size(video,4));
track(:,:,1) = [X2' Y2'];

%% advection

for i = 2:size(video,4)
    if rem(i,10) == 0
        disp(['frame ',num2str(i),' of ',num2str(size(video,4))]);
    end
    for x = 1:size(X,1) %particle x-coordinate
        for y = 1:size(X,2) %particle y-coordinate
            X_ROI = round([X(x,y)-PR/2:X(x,y)+PR/2]); %determine Region Of Interest
            Y_ROI = round([Y(x,y)-PR/2:Y(x,y)+PR/2]);
            if (min(X_ROI>0) && max(X_ROI) <= size(video,1)) && (min(Y_ROI >0) && max(Y_ROI) <= size(video,2))
                Opt_ROI = Vopt(X_ROI,Y_ROI,i,:); % extract optical flow of ROI, TO DO: fix matrix dimensions exceeding
            else
                Opt_ROI = zeros(size(w_av,1),size(w_av,2),1,2);
            end
            Opt_av(x,y,i,1) = sum(sum(w_av.*Opt_ROI(:,:,1,1))); %calculate weighted average (x-direction)
            Opt_av(x,y,i,2) = sum(sum(w_av.*Opt_ROI(:,:,1,2))); %calculate weighted average (y-direction)
            Opt_loc(x,y,i,1)= Vopt(round(X(x,y)),round(Y(x,y)),i,1); %Local particle velocity (x-direction)
            Opt_loc(x,y,i,2)= Vopt(round(X(x,y)),round(Y(x,y)),i,2); %Local particle velocity (y-direction)
        end
    end
    %calculate new particle positions
    X = X+Opt_av(:,:,i,1);
    Y = Y+Opt_av(:,:,i,2);
    X2 = reshape(X,[1,size(X,1)*size(X,2)]);
    Y2 = reshape(Y,[1,size(Y,1)*size(Y,2)]);
    track(:,:,i) = [X2' Y2'];
       [row,col]=find(round(X)>=(size(video,1)-PR));
       if size(row,1)>0
           for idx = 1:size(row) %particle x-coordinate
                X(row(idx),col(idx))=Xint(row(idx),col(idx));
                Y(row(idx),col(idx))=Yint(row(idx),col(idx));
           end
       end
       [row,col]=find(round(X)<=PR);
       if size(row,1)>0
            for idx = 1:size(row) %particle x-coordinate
                X(row(idx),col(idx))=Xint(row(idx),col(idx));
                Y(row(idx),col(idx))=Yint(row(idx),col(idx));
            end
       end
       [row,col]=find(round(Y)>=(size(video,2)-PR));
       if size(row,1)>0
            for idx = 1:size(row) %particle x-coordinate
                X(row(idx),col(idx))=Xint(row(idx),col(idx));
                Y(row(idx),col(idx))=Yint(row(idx),col(idx));
            end
       end
       [row,col]=find(round(Y)<=PR);
       if size(row,1)>0
            for idx = 1:size(row) %particle x-coordinate
                X(row(idx),col(idx))=Xint(row(idx),col(idx));
                Y(row(idx),col(idx))=Yint(row(idx),col(idx));
            end
       end
end

end

